﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using DlhSoft.Windows.Controls;
using DlhSoft.Windows.Data;

namespace GanttChartTaskListViewEffortPartitionsSample
{
    // Represents an effort partitioning enabled task item inheriting from the base TaskItem class.
    class CustomTaskItem : TaskItem
    {
        // Initializes the effort partitioning task item by initializing its schedule.
        public CustomTaskItem()
        {
            UpdateSchedule();
        }

        // Defines the effort interruptions of the task.
        private ObservableCollection<TimeInterval> effortInterruptions;
        public ObservableCollection<TimeInterval> EffortInterruptions
        {
            get { return effortInterruptions; }
            set
            {
                if (value == EffortInterruptions)
                    return;
                if (EffortInterruptions != null)
                    EffortInterruptions.CollectionChanged -= EffortInterruptions_CollectionChanged;
                effortInterruptions = value;
                if (EffortInterruptions != null)
                    EffortInterruptions.CollectionChanged += EffortInterruptions_CollectionChanged;
                UpdateSchedule();
            }
        }

        private void EffortInterruptions_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            UpdateSchedule();
        }

        // Update the Schedule task property value based on its effort partitions.
        private void UpdateSchedule()
        {
            Schedule = GetScheduleFromEffortPartitions();
        }

        // Gets a standard based schedule instance excluding the defined effort interruptions generating effort partitions.
        private Schedule GetScheduleFromEffortPartitions()
        {
            return
                new Schedule("Effort partitions, standard",
                    (DayScheduleProvider)delegate(Date date)
                    {
                        TimeIntervalCollection standardTimeIntervalCollection = Schedule.StandardDayScheduleProvider(date);
                        if (standardTimeIntervalCollection == null || standardTimeIntervalCollection.Count == 0 || EffortInterruptions == null || EffortInterruptions.Count == 0)
                            return standardTimeIntervalCollection;
                        TimeIntervalCollection effortPartitionTimeIntervalCollection = new TimeIntervalCollection();
                        int effortInterruptionCount = EffortInterruptions.Count;
                        for (int i = 0; i < effortInterruptionCount; i++)
                        {
                            TimeInterval effortInterruption = EffortInterruptions[i];
                            if (i == 0)
                            {
                                effortPartitionTimeIntervalCollection.Add(new TimeInterval(DateTime.MinValue, effortInterruption.Start));
                            }
                            else
                            {
                                TimeInterval previousEffortInterruption = EffortInterruptions[i - 1];
                                effortPartitionTimeIntervalCollection.Add(new TimeInterval(previousEffortInterruption.Finish, effortInterruption.Start));
                            }
                        }
                        if (effortInterruptionCount > 0)
                        {
                            TimeInterval lastEffortInterruption = EffortInterruptions[effortInterruptionCount - 1];
                            effortPartitionTimeIntervalCollection.Add(new TimeInterval(lastEffortInterruption.Finish, DateTime.MaxValue));
                        }
                        else
                        {
                            effortPartitionTimeIntervalCollection.Add(new TimeInterval(DateTime.MinValue, DateTime.MaxValue));
                        }
                        TimeIntervalCollection timeIntervalCollection = new TimeIntervalCollection();
                        foreach (TimeInterval effortPartitionTimeInterval in effortPartitionTimeIntervalCollection)
                        {
                            foreach (TimeInterval standardTimeInterval in standardTimeIntervalCollection)
                            {
                                TimeInterval? timeInterval = effortPartitionTimeInterval.Intersect(standardTimeInterval);
                                if (timeInterval != null)
                                    timeIntervalCollection.Add((TimeInterval)timeInterval);
                            }
                        }
                        return timeIntervalCollection;
                    });
        }
    }
}
